/**
 * ChildProcess.h
 * Copyright (C) 2006 Kazuyoshi Kakihara
 *
 * This file is part of dzformatter.
 *
 * dzformatter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#if !defined(AFX_CHILDPROCESS_H__E46F5EF8_895C_4020_8DF7_389EB13CE7F8__INCLUDED_)
#define AFX_CHILDPROCESS_H__E46F5EF8_895C_4020_8DF7_389EB13CE7F8__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CChildProcess {
  LPSTR commandline;
  LPSTR currentdir;
  LPVOID environment;
  PROCESS_INFORMATION pi;
  DWORD dwExitCode;
  HANDLE hStdIn;
  HANDLE hStdOut;
  HANDLE hStdErr;
public:
  CChildProcess()
  {
    commandline = NULL;
    currentdir = NULL;
    environment = NULL;
    dwExitCode = 0;
    hStdIn = NULL;
    hStdOut = NULL;
    hStdErr = NULL;
    ZeroMemory(&pi, sizeof(PROCESS_INFORMATION));
  }
  virtual ~CChildProcess()
  {
    if (commandline) delete [] commandline;
    if (currentdir) delete [] currentdir;
    if (environment) delete [] environment;
    if (pi.hProcess) ::CloseHandle(pi.hProcess);
    if (pi.hThread) ::CloseHandle(pi.hProcess);
  }

  /*!
   * iOptionalj
   * KvɉĎqvZX̕ẂAWóA
   * WG[o͗p̃nhݒ肷邱ƂłB
   * CChildProcessł͂̃nhCloseȂB
   * ĂяoŕKClose邱ƁB
   */
  HRESULT setStdIn(HANDLE hStdIn) { this->hStdIn = hStdIn; return S_OK; }
  HRESULT setStdOut(HANDLE hStdOut) { this->hStdOut = hStdOut; return S_OK; }
  HRESULT setStdErr(HANDLE hStdErr) { this->hStdErr = hStdErr; return S_OK; }

  /*!
   * iK{j
   * sAvP[ṼR}hCݒB
   */
  HRESULT setCommandLine(LPCSTR lpCommandLine)
  {
    HRESULT hr = S_OK;
    if (!lpCommandLine) return HRESULT_FROM_WIN32(ERROR_INVALID_DATA);
    size_t dwLen = ::lstrlen(lpCommandLine);
    dwLen += 1;
    try {
      this->commandline = new CHAR[dwLen];
      ::lstrcpy(this->commandline, lpCommandLine);
    } catch (...) {
      this->commandline = NULL;
      hr = HRESULT_FROM_WIN32(ERROR_OUTOFMEMORY);
    }
    return hr;
  }

  /*!
   * iOptionalj
   * qvZX̊ϐݒ肷B
   * ꂪݒ肳ȂΐevZX̊ϐgpB
   * lpEnvironment \nŋ؂ꂽname=value`̊ϐubNB
   * ubN̍Ōɂ͕Kst邱ƁB
   */
  HRESULT setEnvironment(LPVOID lpEnvironment)
  {
    HRESULT hr = S_OK;
    if (!lpEnvironment) return S_OK;
    size_t dwLen = ::lstrlen((LPCSTR)lpEnvironment);
    dwLen += 2;
    try {
      this->environment = new CHAR[dwLen];
      ::lstrcpy((LPSTR)(this->environment), (LPCSTR)lpEnvironment);
      if (FAILED(hr)) return hr;
      for (LPSTR p = (LPSTR)(this->environment); *p; ++p) {
        if (*p == '\n') *p = '\0';
      }
      return S_OK;
    } catch (...) {
      this->environment = NULL;
      return HRESULT_FROM_WIN32(ERROR_OUTOFMEMORY);
    }
  }
  
  /*!
   * iOptionalj
   * qvZX̃JgfBNgݒ肷B
   */
  HRESULT setCurrentDirectory(LPCSTR lpCurrentDirectory)
  {
    HRESULT hr = S_OK;
    if (!lpCurrentDirectory) return S_OK;
    size_t dwLen = ::lstrlen(lpCurrentDirectory);
    dwLen += 1;
    try {
      this->currentdir = new CHAR[dwLen];
      ::lstrcpy(this->currentdir, lpCurrentDirectory);
    } catch (...) {
      this->currentdir = NULL;
      hr = HRESULT_FROM_WIN32(ERROR_OUTOFMEMORY);
    }
    return hr;
  }

  /*!
   * iK{j
   * SetCommandLineŐݒ肳ꂽR}hsB
   */
  HRESULT execute(BOOL bShow)
  {
    HRESULT hr = S_OK;

    SECURITY_ATTRIBUTES sa;
    ZeroMemory(&sa, sizeof(SECURITY_ATTRIBUTES));
    sa.nLength = sizeof(SECURITY_ATTRIBUTES);
    sa.bInheritHandle = TRUE;

    // ̂ăpCv
    HANDLE hNull = ::CreateFile("nul", GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE, &sa, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
    if (hNull == INVALID_HANDLE_VALUE) {
      hr = HRESULT_FROM_WIN32(::GetLastError());
      return hr;
    }

    // STARTUPINFO ݒ
    STARTUPINFO si;
    ZeroMemory(&si, sizeof(STARTUPINFO));
    si.cb = sizeof(STARTUPINFO);
    si.dwFlags = STARTF_USESTDHANDLES | STARTF_USESHOWWINDOW;
    si.wShowWindow = bShow ? SW_SHOW : SW_HIDE;
    si.hStdInput = hStdIn ? hStdIn : hNull;
    si.hStdOutput = hStdOut ? hStdOut : hNull;
    si.hStdError = hStdErr ? hStdErr : hNull;
    if (!::CreateProcess(NULL, commandline, NULL, NULL, TRUE, 0, environment, currentdir, &si, &pi)) {
      DWORD dwErr = ::GetLastError();
      hr = HRESULT_FROM_WIN32(::GetLastError());
      CloseHandle(hNull);
      return hr;
    }
    CloseHandle(pi.hThread);
    pi.hThread = NULL;
    CloseHandle(hNull);

    return hr;
  }

  /*!
   * iOptionalj
   * qvZX̏I҂B
   * @param ҂ԁi~bj
   * dwMillisecondsINFINITEw肷ƎqvZXI܂ő҂B
   * dwMillisecondso߂ĂIȂꍇ
   * WAIT_TIMEOUTԂB
   */
  DWORD waitForFinish(DWORD dwMilliseconds)
  {
    return ::WaitForSingleObject(pi.hProcess, dwMilliseconds);
  }

  /*!
   * iOptionalj
   * qvZX̏IR[h𓾂B
   * qvZX܂IĂȂꍇSTILL_ACTIVEԂB
   */
  DWORD getExitCode(void)
  {
    DWORD dwExitCode;
    ::GetExitCodeProcess(pi.hProcess, &dwExitCode);
    return dwExitCode;
  }
};

#endif // !defined(AFX_CHILDPROCESS_H__E46F5EF8_895C_4020_8DF7_389EB13CE7F8__INCLUDED_)
