/**
 * DzDocument.cpp
 * Copyright (C) 2006 Kazuyoshi Kakihara
 *
 * This file is part of dzformatter.
 *
 * dzformatter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "stdafx.h"
#include "DzNode.h"
#include "DzDocument.h"

//////////////////////////////////////////////////////////////////////
// \z/
//////////////////////////////////////////////////////////////////////

CDzDocument::CDzDocument()
{

}

CDzDocument::~CDzDocument()
{

}

HRESULT CDzDocument::inc_itr(std::string::const_iterator &itr, const std::string::const_iterator &itr_end)
{
  if (itr == itr_end) return E_FAIL;
  if (::IsDBCSLeadByte(*itr)) {
    ++itr;
    if (itr == itr_end) return E_FAIL;
  }
  ++itr;
  return S_OK;
}

HRESULT CDzDocument::addinc_itr(std::string::const_iterator &itr, const std::string::const_iterator &itr_end, std::string &str)
{
  if (itr == itr_end) return E_FAIL;
  if (::IsDBCSLeadByte(*itr)) {
    std::string::const_iterator itr0 = itr;
    ++itr;
    if (itr == itr_end) return E_FAIL;
    str.append(itr0, itr + 1);
  } else {
    if (*itr == '&') {
      str.append("&amp;");
    } else if (*itr == '<') {
      str.append("&lt;");
    } else if (*itr == '>') {
      str.append("&gt;");
    } else if (*itr == '"') {
      str.append("&quot;");
    } else {
      str.append(itr, itr + 1);
    }
  }
  ++itr;
  return S_OK;
}

std::string CDzDocument::escapeHtml(const std::string &str)
{
  return escapeHtml(str.begin(), str.end());
}

std::string CDzDocument::escapeHtml(const std::string::const_iterator &itr_begin, const std::string::const_iterator &itr_end)
{
  std::string sret;
  for (std::string::const_iterator itr = itr_begin; itr != itr_end; ++itr) {
    if (*itr == '&') {
      sret.append("&amp;");
    } else if (*itr == '<') {
      sret.append("&lt;");
    } else if (*itr == '>') {
      sret.append("&gt;");
    } else if (*itr == '"') {
      sret.append("&quot;");
    } else {
      sret.append(itr, itr + 1);
    }
  }
  return sret;
}

HRESULT CDzDocument::parseInParen(std::string::const_iterator &itr, const std::string::const_iterator &itr_end, CDzNode &dzNode, const std::string &stopstr)
{
  HRESULT hr = S_OK;
  BOOL fClassId = FALSE;
  std::string str;
  while (TRUE) {
    if (*itr == ' ') {
      return E_FAIL;
    } else if (((itr == itr_end) && (stopstr.empty())) ||
               (stopstr.find(*itr) != std::string::npos)) {
      if (!fClassId) {
        dzNode.set_class(str);
      } else {
        dzNode.set_id(str);
      }
      break;
    } else if (itr == itr_end) {
      return E_FAIL;
    } else if (*itr == '#') {
      if (!fClassId) {
        dzNode.set_class(str);
        ++itr;
        fClassId = TRUE;
        str = "";
      } else {
        // #2ȏ㌻ꂽG[
        return E_FAIL;
      }
    }
    if (FAILED(addinc_itr(itr, itr_end, str))) return E_FAIL;
  }
  return hr;
}

HRESULT CDzDocument::parseInBrace(std::string::const_iterator &itr, const std::string::const_iterator &itr_end, CDzNode &dzNode, const std::string &default_cmd, const std::string &stopstr)
{
  HRESULT hr = S_OK;

  // R}hm
  if ((itr != itr_end) &&
      (stopstr.find(*itr) == std::string::npos) &&
      (*itr != ' ') &&
      (*itr != '(')) {
    std::string::const_iterator itr0 = itr;
    while ((itr != itr_end) && 
           (stopstr.find(*itr) == std::string::npos) &&
           (*itr != ' ') &&
           (*itr != '(')) {
      if (FAILED(hr = inc_itr(itr, itr_end))) return hr;
    }
    dzNode.set_cmd(std::string(itr0, itr));
  } else {
    dzNode.set_cmd(default_cmd);
  }

  // class,id̏
  if ((itr != itr_end) && (*itr == '(')) {
    // class,id
    if (FAILED(hr = inc_itr(itr, itr_end))) return hr;
    if (FAILED(hr = parseInParen(itr, itr_end, dzNode, ")"))) return hr;
    if (FAILED(hr = inc_itr(itr, itr_end))) return hr;
  }
  
  // 󔒂̓ǂݔ΂
  while ((itr != itr_end) && 
         (stopstr.find(*itr) == std::string::npos) &&
         (*itr == ' ')) {
    if (FAILED(hr = inc_itr(itr, itr_end))) return hr;
  }

  // vars̏
  while (itr != itr_end) {
    if (stopstr.find(*itr) != std::string::npos) {
      break;
    } else if (*itr == '"') { // double quoted
      if (FAILED(hr = inc_itr(itr, itr_end))) return hr;
      std::string strvar;
      while (true) {
        if (itr == itr_end) return E_FAIL;
        if (*itr == '"') break;
        if (*itr == '\\') {
          if (FAILED(hr = inc_itr(itr, itr_end))) return hr;
          if (itr == itr_end) return E_FAIL;
          if (FAILED(hr = addinc_itr(itr, itr_end, strvar))) return hr;
        } else {
          if (FAILED(hr = addinc_itr(itr, itr_end, strvar))) return hr;
        }
      }
      dzNode.add_vars(strvar);
      if (FAILED(hr = inc_itr(itr, itr_end))) return hr;
    } else { // not quoted
      std::string strvar;
      while (true) {
        if ((itr == itr_end) || 
            (*itr == ' ') || 
            (stopstr.find(*itr) != std::string::npos)) {
          break;
        }
        if (FAILED(hr = addinc_itr(itr, itr_end, strvar))) return hr;
      }
      dzNode.add_vars(strvar);
    }
    while ((itr != itr_end) && (*itr == ' ')) ++itr;
  }
  return hr;
}

HRESULT CDzDocument::str2dzNode(std::string::const_iterator &itr, const std::string::const_iterator &itr_end, CDzNode &dzNode)
{
  HRESULT hr = S_OK;
  CHAR basecmd = *itr;

  // xZo
  int level = 0;
  while ((itr != itr_end) && (*itr == basecmd)) {
    ++itr;
    dzNode.add_level();
  }
  if (itr != itr_end) {
    if (basecmd == '|') {
      // ėpubN̏ꍇ
      if (*itr == '{') {
        // commandL
        // { }ň͂܂ꂽ͂NodeHeaderIuWFNgnhɊi[
        if (FAILED(hr = inc_itr(itr, itr_end))) return hr;
        if (itr == itr_end) return E_FAIL;
        hr = parseInBrace(itr, itr_end, dzNode, "div", "}");
        if (FAILED(hr)) return hr;
        // itr{ }̊O
        if (FAILED(hr = inc_itr(itr, itr_end))) return hr;
        if (itr == itr_end) return E_FAIL;
        m_prediv = dzNode.get_cmd();
      } else {
        if (m_prediv.empty()) {
          dzNode.set_cmd("div");
        } else {
          dzNode.set_cmd(m_prediv);
        }
      }
    } else {
      // \sLubN̏ꍇ
      // m_cmd m
      if (m_exmode == exportmode_html) {
        if (basecmd == '*') dzNode.set_cmd("div");
        else if (basecmd == '-') dzNode.set_cmd("ul");
        else if (basecmd == '>') dzNode.set_cmd("blockquote");
        else if (basecmd == '!') dzNode.set_cmd("dl");
      } else if (m_exmode == exportmode_xml) {
        if (basecmd == '*') dzNode.set_cmd("section");
        else if (basecmd == '-') dzNode.set_cmd("list");
        else if (basecmd == '>') dzNode.set_cmd("blockquote");
        else if (basecmd == '!') dzNode.set_cmd("dlist");
      } else {
        return E_FAIL;
      }

      if ((itr != itr_end) && (*itr == '(')) {
        // sL̎'('΁ANXAID擾
        // ( ) ̓ɃCe[^i߂
        if (FAILED(hr = inc_itr(itr, itr_end))) return hr;
        if (itr == itr_end) return E_FAIL;
        hr = parseInParen(itr, itr_end, dzNode, ")");
        if (FAILED(hr)) return hr;
        // itr( )̊O
        if (FAILED(hr = inc_itr(itr, itr_end))) return hr;
      }
    }
    // R}hw蕔ɕ񂪑ꍇ́AԂɋ󔒂1ȂĂ͂ȂȂ
    if (itr != itr_end) {
      if (*itr != ' ') return -1;
      if (FAILED(hr = inc_itr(itr, itr_end))) return hr;
    }
  }
  return hr;
}

HRESULT CDzDocument::dzNode2XmlStr(CDzNode &dzNode, std::string &str)
{
  str = std::string("<" + dzNode.get_cmd());
  if (!dzNode.get_class().empty()) {
    str += " class=\"" + dzNode.get_class() + "\"";
  }
  if (!dzNode.get_id().empty()) {
    str += " id=\"" + dzNode.get_id() + "\"";
  }

  if (m_exmode == exportmode_xml) {
    if (dzNode.get_vars_length() > 0) {
      for (int i = 0; i < dzNode.get_vars_length(); ++i) {
        char buff[10];
        str += std::string(" param") + ::_itoa(i, buff, 10) + "=\"" + dzNode.get_vars_at(i) + "\"";
      }
    }
  }
  str += ">";
  return S_OK;
}

HRESULT CDzDocument::dzNode2XmlStr2(CDzNode &dzNode, std::string &str)
{
  str = std::string("<" + dzNode.get_cmd());
  if (!dzNode.get_class().empty()) {
    str += " class=\"" + dzNode.get_class() + "\"";
  }
  if (!dzNode.get_id().empty()) {
    str += " id=\"" + dzNode.get_id() + "\"";
  }
  if (dzNode.get_vars_length() > 0) {
    for (int i = 0; i < dzNode.get_vars_length(); ++i) {
      char buff[10];
      str += std::string(" param") + ::_itoa(i, buff, 10) + "=\"" + dzNode.get_vars_at(i) + "\"";
    }
  }
  str += ">";
  return S_OK;
}

HRESULT CDzDocument::parseInLine(std::string::const_iterator &itr, const std::string::const_iterator &itr_end, std::string &str_dst, const std::string &stopstr)
{
  HRESULT hr = S_OK;
  while (itr != itr_end) {
    if ((!stopstr.empty()) && (stopstr.find(*itr) != std::string::npos)) {
      break;
    } else if (*itr == '_') {
      ++itr;
      if (itr == itr_end) return E_FAIL;

      std::string str1;
      if (FAILED(hr = parseInLine(itr, itr_end, str1, "[*{"))) return hr;
      if (*itr == '[') {
        // ruby
        ++itr;
        if (itr == itr_end) return E_FAIL;
        std::string str2;
        while (*itr != ']') {
          if (*itr == '\\') {
            ++itr;
            if (itr == itr_end) return E_FAIL;
          }
          if (FAILED(hr = addinc_itr(itr, itr_end, str2))) return hr;
          if (itr == itr_end) return E_FAIL;
        }
        if (m_exmode == exportmode_html) {
          str_dst += std::string("<ruby><rb>" + escapeHtml(str1) + "</rb><rp>(</rp><rt>" + str2 + "</rt><rp>)</rp></ruby>");
        } else if (m_exmode == exportmode_xml) {
          str_dst += std::string("<ruby param0=\"" + escapeHtml(str2) + "\">" + str1 + "</ruby>");
        } else {
          return E_FAIL;
        }
        ++itr;
      } else if (*itr == '*') { // emph
        str_dst += "<em>" + str1 + "</em>";
        ++itr;
      } else if (*itr == '{') { // general
        CDzNode dzNode;
        ++itr; // '{' ǂݔ΂
        if (FAILED(hr = parseInBrace(itr, itr_end, dzNode, "span", "}"))) return hr;
        ++itr;
        std::string str2;
        if (FAILED(hr = dzNode2XmlStr2(dzNode, str2))) return hr;
        if (m_exmode == exportmode_html) {
          str_dst += std::string("<span class=\"inline_command\">") + escapeHtml(str2) + str1 + "&lt;/" + dzNode.get_cmd() + "&gt;</span>";
        } else if (m_exmode == exportmode_xml) {
          str_dst += str2 + str1 + "</" + dzNode.get_cmd() + ">\r\n";
        } else {
          return E_FAIL;
        }
      } else {
        return E_FAIL;
      }
    } else if (*itr == '\\') {
      ++itr;
      if (itr == itr_end) return E_FAIL;
      if (FAILED(hr = addinc_itr(itr, itr_end, str_dst))) return hr;
    } else {
      if (FAILED(hr = addinc_itr(itr, itr_end, str_dst))) return hr;
    }
  }
  return hr;
}

HRESULT CDzDocument::parseBlankLine(void)
{
  HRESULT hr = S_OK;
  
  while ((m_nodeStack.size() > 0) &&
    (m_nodeStack[m_nodeStack.size() - 1].get_type() != CDzNode::nodetypes::nt_document) &&
    (m_nodeStack[m_nodeStack.size() - 1].get_type() != CDzNode::nodetypes::nt_section)) {
    m_converted.push_back(std::string("</") + m_nodeStack[m_nodeStack.size() - 1].get_cmd() + ">\r\n");
    m_nodeStack.pop_back();
  }

  return hr;
}

HRESULT CDzDocument::parseOneLine(const std::string &str) {
  HRESULT hr = S_OK;
  std::string::const_iterator itr = str.begin();
  const std::string::const_iterator itr_end = str.end();

  ++itr;
  if ((itr == itr_end) ||
    (*itr == ' ') ||
    (*itr == '(')) {
    hr = E_FAIL;
    return hr;
  }
  CDzNode dzNode;
  if (FAILED(hr = parseInBrace(itr, itr_end, dzNode, "div", ""))) return hr;
  if (dzNode.get_cmd() != "div") {
    m_converted.push_back("<div class=\"" + dzNode.get_cmd() + "\">");
  }
  std::string str1;
  if (FAILED(hr = dzNode2XmlStr2(dzNode, str1))) return hr;
  if (m_exmode == exportmode_html) {
    m_converted.push_back(std::string() + "<pre class=\"oneline_command\">" + escapeHtml(str1) + "</" + dzNode.get_cmd() + "></pre>\r\n");
  } else if (m_exmode == exportmode_xml) {
    m_converted.push_back(std::string() + str1 + "</" + dzNode.get_cmd() + ">\r\n");
  } else {
    return E_FAIL;
  }
  if (dzNode.get_cmd() != "div") {
    m_converted.push_back("</div>\r\n");
  }

  return hr;
}

HRESULT CDzDocument::parseSectionLine(const std::string &str) {
  HRESULT hr = S_OK;
  std::string::const_iterator itr = str.begin();
  const std::string::const_iterator itr_end = str.end();
  CDzNode dzNode;

  if (FAILED(hr == str2dzNode(itr, itr_end, dzNode))) return hr;

  // Jg̃ZNVxZo
  int nestlevel = 0;
  for (int i = m_nodeStack.size(); i > 0; --i) {
    if (m_nodeStack[i - 1].get_type() == CDzNode::nodetypes::nt_document) break;
    if (m_nodeStack[i - 1].get_type() == CDzNode::nodetypes::nt_section) ++nestlevel;
  }
  // ʂ̃ZNV͕
  while (nestlevel >= dzNode.get_level()) {
    if (m_nodeStack[m_nodeStack.size() - 1].get_type() == CDzNode::nodetypes::nt_document) return E_FAIL;
    if (m_nodeStack[m_nodeStack.size() - 1].get_type() == CDzNode::nodetypes::nt_section) --nestlevel;
    m_converted.push_back(std::string("</") + m_nodeStack[m_nodeStack.size() - 1].get_cmd() + ">\r\n");
    m_nodeStack.pop_back();
  }
  if (nestlevel < dzNode.get_level() - 1) { // ZNVxɔтꍇ͉ʂ͂
    ++nestlevel;
    if (m_exmode == exportmode_html) {
      char buff[10];
      m_converted.push_back(std::string("<div class=\"section_") + ::_itoa(nestlevel, buff, 10) + "\">\r\n");
      m_nodeStack.push_back(CDzNode("div", "", "", std::vector<std::string>(), CDzNode::nodetypes::nt_section, nestlevel));
    } else if (m_exmode == exportmode_xml) {
      m_converted.push_back(std::string("<section>\r\n"));
      m_nodeStack.push_back(CDzNode("section", "", "", std::vector<std::string>(), CDzNode::nodetypes::nt_section, nestlevel));
    } else {
      return E_FAIL;
    }
  }
  
  // ZNVJn
  {
    ++nestlevel;
    if (m_exmode == exportmode_html) {
      if (dzNode.get_class() == "") {
        CHAR buff[10];
        dzNode.set_class(std::string("section_") + ::_itoa(nestlevel, buff, 10));
      }
    }
    std::string str1;
    if (FAILED(hr = dzNode2XmlStr(dzNode, str1))) return hr;
    m_converted.push_back(str1);
    dzNode.set_type(CDzNode::nodetypes::nt_section);
    m_nodeStack.push_back(dzNode);
  }

  if (itr != itr_end) {
    // heading
    std::string str1;
    if (FAILED(hr = parseInLine(itr, itr_end, str1, ""))) return hr;
    if (m_exmode == exportmode_html) {
      char buff[10];
      m_converted.push_back(std::string("<h") + ::_itoa(nestlevel, buff, 10) + ">" + str1 + "</h" + ::_itoa(nestlevel, buff, 10) + ">\r\n");
    } else if (m_exmode == exportmode_xml) {
      m_converted.push_back(std::string("<heading>") + str1 + "</heading>\r\n");
    } else {
      return E_FAIL;
    }
  }
  return hr;
}
HRESULT CDzDocument::parseDivLine(const std::string &str)
{
  HRESULT hr = S_OK;
  CDzNode dzNode;
  std::string::const_iterator itr = str.begin();
  const std::string::const_iterator itr_end = str.end();

  if (FAILED(hr = str2dzNode(itr, itr_end, dzNode))) return hr;

  // Jg̃XgxZo
  int nestlevel = 0;
  for (int i = m_nodeStack.size(); i > 0; --i) {
    if (m_nodeStack[i - 1].get_type() == CDzNode::nodetypes::nt_document) break;
    if (m_nodeStack[i - 1].get_type() == CDzNode::nodetypes::nt_section) break;
    if (m_nodeStack[i - 1].get_type() == CDzNode::nodetypes::nt_outer) {
      ++nestlevel;
    } else if (m_nodeStack[i - 1].get_type() == CDzNode::nodetypes::nt_inner) {
    } else {
      nestlevel = 0;
    }
  }
  
  // [lXg\͕
  while (nestlevel >= dzNode.get_level()) {
    while (m_nodeStack[m_nodeStack.size() - 1].get_type() != CDzNode::nodetypes::nt_outer) {
      m_converted.push_back(std::string("</") + m_nodeStack[m_nodeStack.size() - 1].get_cmd() + ">\r\n");
      m_nodeStack.pop_back();
      if (m_nodeStack[m_nodeStack.size() - 1].get_type() == CDzNode::nodetypes::nt_document) return E_FAIL;
    }
    if (nestlevel == dzNode.get_level()) break;
    m_converted.push_back(std::string("</") + m_nodeStack[m_nodeStack.size() - 1].get_cmd() + ">\r\n");
    m_nodeStack.pop_back();
    --nestlevel;
    if (m_nodeStack[m_nodeStack.size() - 1].get_type() == CDzNode::nodetypes::nt_document) return E_FAIL;
  }
  // ŌɊJĂlXgouter\擾
  int ni;
  for (ni = m_nodeStack.size(); ni > 0; --ni) {
    if ((m_nodeStack[ni - 1].get_type() == CDzNode::nodetypes::nt_document) ||
        (m_nodeStack[ni - 1].get_type() == CDzNode::nodetypes::nt_section) ||
        (m_nodeStack[ni - 1].get_type() == CDzNode::nodetypes::nt_outer)) break;
  }
  // VKNXJn̏ꍇ͂ɈlXg\AVlXg\Jn
  if ((!dzNode.get_class().empty()) ||
      (!dzNode.get_id().empty()) ||
      ((nestlevel == dzNode.get_level()) && (dzNode.get_cmd() != m_nodeStack[ni - 1].get_cmd()))) {
    while (nestlevel == dzNode.get_level()) {
      m_converted.push_back(std::string("</") + m_nodeStack[m_nodeStack.size() - 1].get_cmd() + ">\r\n");
      if (m_nodeStack[m_nodeStack.size() - 1].get_type() == CDzNode::nodetypes::nt_outer) {
        m_nodeStack.pop_back();
        --nestlevel;
        if (nestlevel < dzNode.get_level()) break;
      }
      m_nodeStack.pop_back();
    }
    std::string str1;
    dzNode2XmlStr(dzNode, str1);
    m_converted.push_back(str1);

    CDzNode dzNode1(dzNode);
    dzNode.set_class("");
    dzNode.set_id("");
    dzNode.set_vars(std::vector<std::string>());
    m_nodeStack.push_back(dzNode1);
    ++nestlevel;
  }
  if (nestlevel == dzNode.get_level() - 1) {
    std::string str1;
    dzNode2XmlStr(dzNode, str1);
    m_converted.push_back(str1);

    m_nodeStack.push_back(CDzNode(dzNode.get_cmd(), "", "", std::vector<std::string>(), CDzNode::nodetypes::nt_outer, dzNode.get_level()));
    ++nestlevel;
  }
  if (nestlevel < dzNode.get_level() - 1) { // Xgxɔтꍇ̓G[
    return E_FAIL;
  }

  // Xgeꍇ͓eǉ
  if (itr != str.end()) {
    if (m_exmode == exportmode_html) {
      if (dzNode.get_cmd() == "dl") { // dlist̏ꍇ
        std::string str1;
        std::string::const_iterator itr_end = str.end();
        if (FAILED(hr = parseInLine(itr, itr_end, str1, ":"))) return hr;
        m_converted.push_back(std::string("<dt>") + str1 + "</dt>\r\n");
        if (FAILED(hr = inc_itr(itr, itr_end))) return hr;
      }
      if (dzNode.get_cmd() == "ul") {
        dzNode.set_cmd("li");
      } else if (dzNode.get_cmd() == "blockquote") {
        dzNode.set_cmd("p");
      } else if (dzNode.get_cmd() == "dl") {
        dzNode.set_cmd("dd");
      } else {
        dzNode.set_cmd("div");
      }
    } else if (m_exmode == exportmode_xml) {
      if (dzNode.get_cmd() == "dlist") { // dlist̏ꍇ
        std::string str1;
        std::string::const_iterator itr_end = str.end();
        if (FAILED(hr = parseInLine(itr, itr_end, str1, ":"))) return hr;
        m_converted.push_back(std::string("<dterm>") + str1 + "</dterm>\r\n");
        if (FAILED(hr = inc_itr(itr, itr_end))) return hr;
      }
      if (dzNode.get_cmd() == "list") {
        dzNode.set_cmd("litem");
      } else if (dzNode.get_cmd() == "blockquote") {
        dzNode.set_cmd("blockquote_inner");
      } else if (dzNode.get_cmd() == "dlist") {
        dzNode.set_cmd("ddesc");
      } else {
        dzNode.set_cmd("div_inner");
      }
    } else {
      return E_FAIL;
    }
    std::string str1;
    std::string::const_iterator itr_end = str.end();
    if (FAILED(hr == parseInLine(itr, itr_end, str1, ""))) return hr;
    m_converted.push_back(std::string("<") + dzNode.get_cmd() + ">" + str1);
    dzNode.set_type(CDzNode::nodetypes::nt_inner);
    m_nodeStack.push_back(dzNode);
  }
  return hr;

}

HRESULT CDzDocument::parseDefaultLine(const std::string &str)
{
  HRESULT hr = S_OK;
  // ZNVȊÕubN
  while ((m_nodeStack.size() > 0) &&
    (m_nodeStack[m_nodeStack.size() - 1].get_type() != CDzNode::nodetypes::nt_document) &&
    (m_nodeStack[m_nodeStack.size() - 1].get_type() != CDzNode::nodetypes::nt_section)) {
    m_converted.push_back(std::string("</") + m_nodeStack[m_nodeStack.size() - 1].get_cmd() + ">\r\n");
    m_nodeStack.pop_back();
  }
  std::string str1;
  std::string::const_iterator itr = str.begin();
  const std::string::const_iterator itr_end = str.end();
  if (FAILED(hr = parseInLine(itr, itr_end, str1, ""))) return hr;
  if (m_exmode == exportmode_html) {
    m_converted.push_back(std::string("<p>" + str1 + "</p>\r\n"));
  } else if (m_exmode == exportmode_xml) {
    m_converted.push_back(std::string("<paragraph>" + str1 + "</paragraph>\r\n"));
  }
  return hr;

}

HRESULT CDzDocument::parseIncompleteLine(const std::string &str)
{
  HRESULT hr = S_OK;
  // ZNVȊÕubN
  while ((m_nodeStack.size() > 0) &&
    (m_nodeStack[m_nodeStack.size() - 1].get_type() != CDzNode::nodetypes::nt_document) &&
    (m_nodeStack[m_nodeStack.size() - 1].get_type() != CDzNode::nodetypes::nt_section)) {
    m_converted.push_back(std::string("</") + m_nodeStack[m_nodeStack.size() - 1].get_cmd() + ">\r\n");
    m_nodeStack.pop_back();
  }
  m_converted.push_back(std::string("<pre class=\"incomplete\">") + escapeHtml(str) + "</pre>\r\n");
  return hr;
}

HRESULT CDzDocument::parseLine(const std::string &str)
{
  HRESULT hr = S_OK;

  // s̏ꍇ
  if ((str.empty()) || (str[0] == '\r') || (str[0] == '\n')) {
    hr = parseBlankLine();
    return hr;
  }

  if (str[0] != '|') {
    m_prediv = "";
  }
  switch (str[0]) {
  case ';': // Rg
    break;
  case '#': // vvZbT
    break;
  case ':': // 1sR}h
    hr = parseOneLine(str);
    if (FAILED(hr)) {
      parseIncompleteLine(str);
      hr = S_OK;
      return hr;
    }
    break;
  case '*': // section
    hr = parseSectionLine(str);
    if (FAILED(hr)) {
      parseIncompleteLine(str);
      hr = S_OK;
      return hr;
    }
    break;
  case '-':
  case '>':
  case '!':
  case '|':
    hr = parseDivLine(str);
    if (FAILED(hr)) {
      parseIncompleteLine(str);
      hr = S_OK;
      return hr;
    }
    break;
  default:
    hr = parseDefaultLine(str);
    if (FAILED(hr)) {
      parseIncompleteLine(str);
      hr = S_OK;
      return hr;
    }
  }
  return hr;
}

HRESULT CDzDocument::export(std::string &str)
{
  HRESULT hr = S_OK;
  std::vector<std::string>::iterator itr;
  m_converted.clear();
  if (m_exmode == exportmode_xml) {
    m_converted.push_back("<?xml version=\"1.0\" encoding=\"Shift_JIS\"?>");
    m_converted.push_back("<document>");
  }
  for (itr = m_rawArray.begin(); itr != m_rawArray.end(); ++itr) {
    if (FAILED(hr = parseLine(*itr))) break;
  }
  while (m_nodeStack.size() > 0) {
    m_converted.push_back(std::string() + "</" + m_nodeStack[m_nodeStack.size() - 1].get_cmd() + ">\r\n");
    m_nodeStack.pop_back();
  }
  if (m_exmode == exportmode_xml) {
    m_converted.push_back("</document>\r\n");
  }
  str = std::accumulate(m_converted.begin(), m_converted.end(), std::string(""));
  return hr;
}

HRESULT CDzDocument::loadText(const std::string &str)
{
  HRESULT hr = S_OK;
  m_rawArray.clear();
  std::string::const_iterator itr_0 = str.begin(), itr;
  for (itr = str.begin(); itr != str.end(); ++itr) {
    if ((*itr == '\r') || (*itr == '\n')) {
      m_rawArray.push_back(std::string(itr_0, itr));
      if (*itr == '\r') ++itr;
      itr_0 = itr + 1;
    }
  }
  if (itr_0 != itr) {
    m_rawArray.push_back(std::string(itr_0, itr));
  }
  return hr;
}

HRESULT CDzDocument::exportAsHTML(std::string &str)
{
  HRESULT hr = S_OK;
  m_exmode = exportmode_html;
  hr = export(str);
  return hr;
}

HRESULT CDzDocument::exportAsXML(std::string &str)
{
  HRESULT hr = S_OK;
  m_exmode = exportmode_xml;
  hr = export(str);
  return hr;
}

