/**
 * DzDocument.h
 * Copyright (C) 2006 Kazuyoshi Kakihara
 *
 * This file is part of dzformatter.
 *
 * dzformatter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#if !defined(AFX_DZDOCUMENT_H__D337B92A_DA5B_4CE6_921E_A4704E70A3F2__INCLUDED_)
#define AFX_DZDOCUMENT_H__D337B92A_DA5B_4CE6_921E_A4704E70A3F2__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CDzDocument  
{
private:
  std::vector<std::string> m_rawArray;
  std::vector<std::string> m_converted;

  std::vector<CDzNode> m_nodeStack;
  std::string m_prediv;

  HRESULT inc_itr(std::string::const_iterator &itr, const std::string::const_iterator &itr_end);
  HRESULT addinc_itr(std::string::const_iterator &itr, const std::string::const_iterator &itr_end, std::string &str);
  std::string escapeHtml(const std::string &str);
  std::string escapeHtml(const std::string::const_iterator &itr, const std::string::const_iterator &itr_end);

  HRESULT str2dzNode(std::string::const_iterator &itr, const std::string::const_iterator &itr_end, CDzNode &dzNode);
  HRESULT dzNode2XmlStr(CDzNode &dzNode, std::string &str);
  HRESULT dzNode2XmlStr2(CDzNode &dzNode, std::string &str);

  HRESULT parseInParen(std::string::const_iterator &itr, const std::string::const_iterator &itr_end, CDzNode &dzNode, const std::string &stopstr = "");
  HRESULT parseInBrace(std::string::const_iterator &itr, const std::string::const_iterator &itr_end, CDzNode &dzNode, const std::string &default_cmd, const std::string &stopstr);
  HRESULT parseInLine(std::string::const_iterator &itr, const std::string::const_iterator &itr_end, std::string &str_dst, const std::string &stopstr);

  HRESULT parseBlankLine(void);
  HRESULT parseOneLine(const std::string &str);
  HRESULT parseSectionLine(const std::string &str);
  HRESULT parseDivLine(const std::string &str);
  HRESULT parseDefaultLine(const std::string &str);
  HRESULT parseIncompleteLine(const std::string &str);

  HRESULT parseLine(const std::string &str);

  enum exportmode {
    exportmode_html = 0,
    exportmode_xml = 1
  };
  exportmode m_exmode;
  HRESULT export(std::string &str);
public:
	CDzDocument();
	virtual ~CDzDocument();
  HRESULT loadText(const std::string &str);
  HRESULT exportAsHTML(std::string &str);
  HRESULT exportAsXML(std::string &str);
};

#endif // !defined(AFX_DZDOCUMENT_H__D337B92A_DA5B_4CE6_921E_A4704E70A3F2__INCLUDED_)
