/**
 * PipeThread.h
 * Copyright (C) 2006 Kazuyoshi Kakihara
 *
 * This file is part of dzformatter.
 *
 * dzformatter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#if !defined(AFX_PIPETHREAD_H__719502A3_54DA_4B7C_835D_19FB72F167A2__INCLUDED_)
#define AFX_PIPETHREAD_H__719502A3_54DA_4B7C_835D_19FB72F167A2__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CPipeThread {
  BOOL bRead;
  DWORD dwExitCode;
  HANDLE hChild;
  HANDLE hParent;
  HANDLE hThread;
  unsigned threadaddr;
public:
  CPipeThread()
  {
    bRead = FALSE;
    dwExitCode = 0;
    hChild = NULL;
    hParent = NULL;
    hThread = NULL;
    threadaddr = 0;
    arg = NULL;
  }
  
  virtual ~CPipeThread()
  {
    if (hChild) ::CloseHandle(hChild);
    if (hParent) ::CloseHandle(hParent);
    if (hThread) ::CloseHandle(hThread);
  }

  void *arg;

  HRESULT prepare_read(void)
  {
    HRESULT hr = S_OK;

    if (hChild || hParent) {
      hr = HRESULT_FROM_WIN32(ERROR_INVALID_FUNCTION);
      return hr;
    }

    this->bRead = TRUE;

    // pipe쐬
    SECURITY_ATTRIBUTES sa;
    ZeroMemory(&sa, sizeof(SECURITY_ATTRIBUTES));
    sa.nLength = sizeof(SECURITY_ATTRIBUTES);
    sa.bInheritHandle = TRUE;

    HANDLE hProcess = ::GetCurrentProcess();
    HANDLE hRead;
    if (!::CreatePipe(&hRead, &hChild, &sa, 0)) {
      hr = HRESULT_FROM_WIN32(::GetLastError());
      return hr;
    }
    if (!::DuplicateHandle(hProcess, hRead, hProcess, &hParent, 0, FALSE, DUPLICATE_SAME_ACCESS)) {
      hr = HRESULT_FROM_WIN32(::GetLastError());
      ::CloseHandle(hRead);
      ::CloseHandle(hChild);
      hChild = NULL;
      return hr;
    }
    ::CloseHandle(hRead);

    return hr;
  }
  
  HRESULT prepare_write(void)
  {
    HRESULT hr = S_OK;

    if (hChild || hParent) {
      hr = HRESULT_FROM_WIN32(ERROR_INVALID_FUNCTION);
      return hr;
    }

    this->bRead = FALSE;

    // pipe쐬
    SECURITY_ATTRIBUTES sa;
    ZeroMemory(&sa, sizeof(SECURITY_ATTRIBUTES));
    sa.nLength = sizeof(SECURITY_ATTRIBUTES);
    sa.bInheritHandle = TRUE;

    HANDLE hProcess = ::GetCurrentProcess();
    HANDLE hWrite;
    if (!::CreatePipe(&hChild, &hWrite, &sa, 0)) {
      hr = HRESULT_FROM_WIN32(::GetLastError());
      return hr;
    }
    if (!::DuplicateHandle(hProcess, hWrite, hProcess, &hParent, 0, FALSE, DUPLICATE_SAME_ACCESS)) {
      hr = HRESULT_FROM_WIN32(::GetLastError());
      ::CloseHandle(hWrite);
      ::CloseHandle(hChild);
      hChild = NULL;
      return hr;
    }
    ::CloseHandle(hWrite);

    return hr;
  }
  
  HANDLE get_hChild(void) { return hChild; }
  
  HANDLE get_hParent(void) { return hParent; }
  
  HANDLE get_hThread(void) { return hThread; }
  
  void close_hChild(void)
  {
    ::CloseHandle(hChild);
    hChild = NULL;
    return;
  }
  
  void close_hParent(void)
  {
    ::CloseHandle(hParent);
    hParent = NULL;
    return;
  }
  
  HANDLE execute(unsigned (__stdcall procThread)(void *))
  {
    hThread = reinterpret_cast<HANDLE>(::_beginthreadex(NULL, 0, procThread, this, 0, &threadaddr));
    return hThread;
  }
  
  DWORD getExitCode(void)
  {
    DWORD dwCode = 0;
    ::GetExitCodeThread(hThread, &dwCode);
    return dwCode;
  }
};

#endif // !defined(AFX_PIPETHREAD_H__719502A3_54DA_4B7C_835D_19FB72F167A2__INCLUDED_)
