/**
 * MainFrm.h
 * Copyright (C) 2006 Kazuyoshi Kakihara
 *
 * This file is part of dzformatter.
 *
 * dzformatter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#if !defined(AFX_MAINFRM_H__07CD2330_35DA_4260_8287_79CBFE15C4E0__INCLUDED_)
#define AFX_MAINFRM_H__07CD2330_35DA_4260_8287_79CBFE15C4E0__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#ifdef FINDREPLACE_ENABLED
#undef FINDREPLACE_ENABLED
#endif

#define ID_USER_MENU_COMMAND 3280
#define EXT_EDITOR_CONFIG_FILENAME "EDITOR"
#define SCRIPT_CONFIG_FILENAME "script.cfg"
#define DUMMY_TEXT_FILENAME "LICENSE.TXT"
#define INDEX_HTML_FILENAME "index.html"

class CMainFrame : public CFrameWindowImpl<CMainFrame>, public CUpdateUI<CMainFrame>,
    public CMessageFilter, public CIdleHandler
{
private:
  CComPtr<IWebBrowser2> m_pBrowser;
  std::vector<CDzScript> m_scripts;
  std::string m_exteditor;
#ifdef FINDREPLACE_ENABELD
  std::string m_findstr;
  std::string m_replacestr;
#endif
  BOOL m_autopreview;
public:
  DECLARE_FRAME_WND_CLASS(NULL, IDR_MAINFRAME)

  CSplitterWindow  m_wndSplitter;
  CRichEditView m_viewRichEdit;
  CBrowserView m_viewBrowser;
  UINT m_timer;
  BOOL m_fchanged;
  BOOL m_ffindreplaceopen;

  std::string m_filename;

  virtual BOOL PreTranslateMessage(MSG* pMsg)
  {
    if(CFrameWindowImpl<CMainFrame>::PreTranslateMessage(pMsg))
        return TRUE;

    // yCPreTranslateMessageĂяo
    if(m_viewRichEdit.PreTranslateMessage(pMsg))
        return TRUE;

    // EyCPreTranslateMessageĂяo
    if (m_viewBrowser.PreTranslateMessage(pMsg))
      return TRUE;

    return FALSE;
//    return CFrameWindowImpl<CMainFrame>::PreTranslateMessage(pMsg);
  }

  virtual BOOL OnIdle()
  {
//    UIUpdateToolBar();
    return FALSE;
  }

  BEGIN_UPDATE_UI_MAP(CMainFrame)
    UPDATE_ELEMENT(ID_OPTION_AUTO_PREVIEW, UPDUI_MENUPOPUP)
  END_UPDATE_UI_MAP()

  BEGIN_MSG_MAP_EX(CMainFrame)
    MSG_WM_CREATE(OnCreate)
    MESSAGE_HANDLER_EX(WM_CLOSE, OnClose)
    MESSAGE_HANDLER_EX(WM_DESTROY, OnDestroy)
    MESSAGE_HANDLER_EX(WM_TIMER, OnTimer)
#ifdef FINDREPLACE_ENABELD
    MESSAGE_HANDLER_EX(CFindReplaceDialog::GetFindReplaceMsg(), OnFindReplace);
#endif
    COMMAND_ID_HANDLER_EX(ID_APP_EXIT, OnFileExit)
    COMMAND_ID_HANDLER_EX(ID_FILE_NEW, OnFileNew)
    COMMAND_ID_HANDLER_EX(ID_FILE_OPEN, OnFileOpen)
    COMMAND_ID_HANDLER_EX(ID_FILE_SAVE, OnFileSave)
    COMMAND_ID_HANDLER_EX(ID_FILE_SAVE_AS, OnFileSaveAs)
    COMMAND_ID_HANDLER_EX(ID_EDIT_CUT, OnEditCut)
    COMMAND_ID_HANDLER_EX(ID_EDIT_COPY, OnEditCopy)
    COMMAND_ID_HANDLER_EX(ID_EDIT_PASTE, OnEditPaste)
    COMMAND_ID_HANDLER_EX(ID_EDIT_UNDO, OnEditUndo)
    COMMAND_ID_HANDLER_EX(ID_EDIT_REDO, OnEditRedo)
#ifdef FINDREPLACE_ENABELD
    COMMAND_ID_HANDLER_EX(ID_EDIT_FIND, OnEditFind)
    COMMAND_ID_HANDLER_EX(ID_EDIT_REPLACE, OnEditReplace)
#endif
    COMMAND_ID_HANDLER_EX(ID_EDIT_SELALL, OnEditSelAll)
    COMMAND_ID_HANDLER_EX(ID_EDIT_EXT_EDITOR, OnEditExtEditor)
    COMMAND_ID_HANDLER_EX(ID_VIEW_CONVERT, OnViewConvert)
/*
    COMMAND_ID_HANDLER_EX(ID_VIEW_TOOLBAR, OnViewToolBar)
    COMMAND_ID_HANDLER_EX(ID_VIEW_STATUS_BAR, OnViewStatusBar)
*/
    COMMAND_ID_HANDLER_EX(ID_OPTION_EXT_EDITOR, OnOptionExtEditor)
    COMMAND_ID_HANDLER_EX(ID_OPTION_AUTO_PREVIEW, OnOptionAutoPreview)
    COMMAND_ID_HANDLER_EX(ID_HELP_CONTENT, OnHelpContent)
    COMMAND_ID_HANDLER_EX(ID_APP_ABOUT, OnAppAbout)
    COMMAND_CODE_HANDLER_EX(EN_CHANGE, OnChange)
    CHAIN_MSG_MAP(CUpdateUI<CMainFrame>)
    MESSAGE_HANDLER_EX(WM_COMMAND, OnCommand)
    MESSAGE_HANDLER_EX(WM_INITMENUPOPUP, OnInitMenuPopup)
    CHAIN_MSG_MAP(CFrameWindowImpl<CMainFrame>)
  END_MSG_MAP()

// Handler prototypes (uncomment arguments if needed):
//  LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//  LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//  LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

  HRESULT waitReadyStateComplete(VOID)
  {
    HRESULT hr = S_OK;
    VARIANT_BOOL bBusy = VARIANT_TRUE;
    while (TRUE) {
      m_pBrowser->get_Busy(&bBusy);
      if (bBusy == VARIANT_FALSE) break;
      MSG msg;
      if (::PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
        ::TranslateMessage(&msg);
        ::DispatchMessage(&msg);
      }
    }

    READYSTATE state;
    while (TRUE) {
      m_pBrowser->get_ReadyState(&state);
      if (state == READYSTATE_COMPLETE) break;
      MSG msg;
      if (::PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
        ::TranslateMessage(&msg);
        ::DispatchMessage(&msg);
      }
    }
    return hr;
  }

  HRESULT doLoadScriptInit(void)
  {
    CHAR szFileName[MAX_PATH + 1];
    ::GetModuleFileName(NULL, szFileName, MAX_PATH);
    *(::PathFindFileName(szFileName)) = '\0';
    if (::lstrlen(szFileName) + ::lstrlen(SCRIPT_CONFIG_FILENAME) > MAX_PATH) return E_FAIL;
    ::lstrcat(szFileName, SCRIPT_CONFIG_FILENAME);
    
    HANDLE hFile = ::CreateFile(szFileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if (hFile == INVALID_HANDLE_VALUE) return E_FAIL;

    std::string strbuff;
    DWORD dwRead;
    CHAR buff[4096 + 1];
    while (1) {
      if (!::ReadFile(hFile, buff, 4096, &dwRead, NULL)) {
        ::CloseHandle(hFile);
        return E_FAIL;
      }
      strbuff.append(buff, buff + dwRead);
      if (dwRead < 4096) break;
    }
    ::CloseHandle(hFile);

    std::vector<std::string> scripts;
    { // ǂݍ񂾃t@C\r\nŕ
      std::string::const_iterator itr0 = strbuff.begin();
      for (std::string::const_iterator itr = itr0; itr != strbuff.end(); ++itr) {
        if (*itr == '\r')  {
          if (*itr0 != '#') {
            CDzScript dzScript;
            if (FAILED(dzScript.set_script(std::string(itr0, itr), ID_USER_MENU_COMMAND + m_scripts.size()))) return E_FAIL;
            m_scripts.push_back(dzScript);
          }
          ++itr;
          if (itr == strbuff.end()) break;
          itr0 = itr + 1;
        } else if (*itr == '\n') {
          if (*itr != '\n') {
            CDzScript dzScript;
            if (FAILED(dzScript.set_script(std::string(itr0, itr), ID_USER_MENU_COMMAND + m_scripts.size()))) return E_FAIL;
            m_scripts.push_back(dzScript);
          }
          itr0 = itr + 1;
        }
      }
    }
    return S_OK;
  }

  LRESULT OnCreate(LPCREATESTRUCT lpcs)
  {
    if (FAILED(doLoadScriptInit())) {
      MessageBox("Failed to load script.cfg", "Error", MB_OK);
      return -1;
    }

    CMenu menubar = GetMenu();
    CMenu popup = menubar.GetSubMenu(2);
    for (std::vector<CDzScript>::iterator itr = m_scripts.begin(); itr != m_scripts.end(); ++itr) {
      popup.AppendMenu(MF_ENABLED | MF_STRING, (*itr).get_script_id(), (*itr).get_account().c_str());
    }

    CreateSimpleToolBar();

    CreateSimpleStatusBar();

    UIAddToolBar(m_hWndToolBar);

    // Xvb^EBhE쐬
    m_wndSplitter.Create (m_hWnd, rcDefault, NULL,
        WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS);

    // Xvb^EBhEgX^Cݒ
    m_wndSplitter.SetSplitterExtendedStyle(0);

    // yC̃r[EBhE쐬
/*
    m_viewRichEdit.Create(m_wndSplitter, rcDefault, NULL,
      WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
      WS_HSCROLL | WS_VSCROLL | ES_AUTOHSCROLL | ES_AUTOVSCROLL |
      ES_MULTILINE | ES_NOHIDESEL | ES_SAVESEL,
      WS_EX_CLIENTEDGE);
*/
    m_viewRichEdit.Create(m_wndSplitter, rcDefault, NULL,
      WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
      WS_VSCROLL | ES_AUTOVSCROLL |
      ES_MULTILINE | ES_NOHIDESEL | ES_SAVESEL,
      WS_EX_CLIENTEDGE);
    m_wndSplitter.SetSplitterPane(SPLIT_PANE_LEFT, m_viewRichEdit);

    // EyC̃r[EBhE쐬
    CHAR szFileName[MAX_PATH + 1];
    ::GetModuleFileName(NULL, szFileName, MAX_PATH);
    *(::PathFindFileName(szFileName)) = '\0';
    ::lstrcat(szFileName, INDEX_HTML_FILENAME);
    m_viewBrowser.Create(m_wndSplitter, rcDefault, szFileName,
      WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
      WS_HSCROLL | WS_VSCROLL, WS_EX_CLIENTEDGE);
    if (FAILED(m_viewBrowser.QueryControl(&m_pBrowser))) return -1;
    waitReadyStateComplete();

    m_wndSplitter.SetSplitterPane(SPLIT_PANE_RIGHT, m_viewBrowser);

    m_hWndClient = m_wndSplitter;
    UpdateLayout();

    // o[̈ʒuݒ
    RECT rc;
    GetClientRect(&rc);
    m_wndSplitter.SetSplitterPos((rc.right - rc.left) * 1 / 2);

    // ^C}[
    m_timer = ::SetTimer(this->m_hWnd, 1, 1000, 0);
    m_fchanged = FALSE;

#ifdef FINDREPLACE_ENABELD
    // u_CAO͊JĂȂ
    m_ffindreplaceopen = FALSE;
#endif

    // OGfB^ݒt@Cǂݍ
    {
      CHAR szFileName[MAX_PATH + 1];
      *(::PathFindFileName(szFileName)) = '\0';
      ::lstrcat(szFileName, EXT_EDITOR_CONFIG_FILENAME);
      HANDLE hFile = ::CreateFile(szFileName, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
      if (hFile != INVALID_HANDLE_VALUE) {
        DWORD dwRead;
        CHAR szEditor[MAX_PATH + 1];
        if(::ReadFile(hFile, szEditor, MAX_PATH, &dwRead, NULL)) {
          if ((dwRead > 0) && (dwRead < MAX_PATH)) {
            m_exteditor = szEditor;
          }
        }
        ::CloseHandle(hFile);
      }
    }

    // vr[ON
    m_autopreview = TRUE;
    UISetCheck(ID_OPTION_AUTO_PREVIEW, 1);

    // register object for message filtering and idle updates
    CMessageLoop* pLoop = _Module.GetMessageLoop();
    ATLASSERT(pLoop != NULL);
    pLoop->AddMessageFilter(this);
    pLoop->AddIdleHandler(this);

    return 0;
  }

  LRESULT OnTimer(UINT uMsg, WPARAM wParam, LPARAM lParam)
  {
    if ((m_fchanged) && (m_autopreview)) {
      updateBrowser();
      m_fchanged = FALSE;
    }
    return 0;
  }

  LRESULT OnInitMenuPopup(UINT uMsg, WPARAM wParam, LPARAM lParam)
  {
    HMENU hMenu = reinterpret_cast<HMENU>(wParam);
    CHARRANGE cr;

    m_viewRichEdit.GetSel(cr);
    if (cr.cpMin == cr.cpMax) {
      ::EnableMenuItem(hMenu, ID_EDIT_CUT, MF_GRAYED | MF_BYCOMMAND);
      ::EnableMenuItem(hMenu, ID_EDIT_COPY, MF_GRAYED | MF_BYCOMMAND);
    } else {
      ::EnableMenuItem(hMenu, ID_EDIT_CUT, MF_ENABLED | MF_BYCOMMAND);
      ::EnableMenuItem(hMenu, ID_EDIT_COPY, MF_ENABLED | MF_BYCOMMAND);
    }
    if (m_viewRichEdit.CanUndo()) {
      ::EnableMenuItem(hMenu, ID_EDIT_UNDO, MF_ENABLED | MF_BYCOMMAND);
    } else {
      ::EnableMenuItem(hMenu, ID_EDIT_UNDO, MF_GRAYED | MF_BYCOMMAND);
    }
    if (m_viewRichEdit.CanRedo()) {
      ::EnableMenuItem(hMenu, ID_EDIT_REDO, MF_ENABLED | MF_BYCOMMAND);
    } else {
      ::EnableMenuItem(hMenu, ID_EDIT_REDO, MF_GRAYED | MF_BYCOMMAND);
    }
    if (m_viewRichEdit.CanPaste()) {
      ::EnableMenuItem(hMenu, ID_EDIT_PASTE, MF_ENABLED | MF_BYCOMMAND);
    } else {
      ::EnableMenuItem(hMenu, ID_EDIT_PASTE, MF_GRAYED | MF_BYCOMMAND);
    }
    return 0;
  }

  LRESULT OnClose(UINT uMsg, WPARAM wParam, LPARAM lParam)
  {
    LRESULT lResult = 0;
    // ɕύXꍇ͕ۑmF
    if (m_viewRichEdit.GetModify()) {
      int iRet;
      iRet = MessageBox("ύXۑ܂?", "mF", MB_YESNOCANCEL);
      if (iRet == IDYES) {
        if (m_filename.empty()) {
          if (!doGetFileName()) {
            return 0;
          }
          doSaveFile(m_filename.c_str());
          m_viewRichEdit.SetModify(FALSE);
        } else {
          doSaveFile(m_filename.c_str());
          m_viewRichEdit.SetModify(FALSE);
        }
        DestroyWindow();
      } else if (iRet == IDNO) {
        // j
        m_filename = "";
        m_viewRichEdit.SetWindowText("");
        m_viewRichEdit.EmptyUndoBuffer();
        m_viewRichEdit.SetModify(FALSE);
        DestroyWindow();
      } else if (iRet == IDCANCEL) {
        // ҏWs
      }
    } else {
       DestroyWindow();
    }
    return 0;
  }

  LRESULT OnDestroy(UINT uMsg, WPARAM wParam, LPARAM lParam)
  {
    ::KillTimer(this->m_hWnd, m_timer);
    PostQuitMessage(0);
    return 0; 
  }

  void OnFileExit(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    SendMessage(WM_CLOSE, 0, 0L);
    return;
  }

  void OnFileNew(UINT uNotifyCode, int nID, HWND hWndCtl)
  {

    // ɕύXꍇ͕ۑmF
    if (m_viewRichEdit.GetModify()) {
      int iRet;
      iRet = MessageBox("ύXۑ܂?", "mF", MB_YESNOCANCEL);
      if (iRet == IDYES) {
        if (m_filename.empty()) {
          OnFileSaveAs(uNotifyCode, nID, hWndCtl);
        } else {
          OnFileSave(uNotifyCode, nID, hWndCtl);
        }
      } else if (iRet == IDNO) {
        // j
        m_filename = "";
        m_viewRichEdit.SetWindowText("");
        m_viewRichEdit.EmptyUndoBuffer();
        m_viewRichEdit.SetModify(FALSE);
      } else if (iRet == IDCANCEL) {
        // ҏWs
      }
    } else {
      // j
      m_filename = "";
      m_viewRichEdit.SetWindowText("");
      m_viewRichEdit.EmptyUndoBuffer();
      m_viewRichEdit.SetModify(FALSE);
    }
    return;
  }

  static DWORD CALLBACK procReadFile(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb)
  {
    ::ReadFile(reinterpret_cast<HANDLE>(dwCookie), pbBuff, cb, reinterpret_cast<LPDWORD>(pcb), NULL);
    return FALSE;
  }

  HRESULT doLoadFile(LPCSTR lpcszFileName)
  {
    HANDLE hFile = ::CreateFile(lpcszFileName, GENERIC_READ, 0, NULL, OPEN_EXISTING,
                                FILE_ATTRIBUTE_NORMAL, NULL);
    if (hFile == INVALID_HANDLE_VALUE) {
      return HRESULT_FROM_WIN32(::GetLastError());
    }
    EDITSTREAM es;
    es.dwCookie = reinterpret_cast<DWORD>(hFile);
    es.dwError = 0;
    es.pfnCallback = procReadFile;
    m_viewRichEdit.StreamIn(SF_TEXT, es);
    _ASSERT(!es.dwError);
    ::CloseHandle(hFile);
    return S_OK;
  }

  void OnFileOpen(UINT uNotifyCode, int nID, HWND hWndCtl)
  {

    // ɕύXꍇ͕ۑmF
    if (m_viewRichEdit.GetModify()) {
      int iRet;
      iRet = MessageBox("ύXۑ܂?", "mF", MB_YESNOCANCEL);
      if ((iRet == IDYES) || (iRet == IDNO)) {
        if (iRet == IDYES) {
          if (m_filename.empty()) {
            OnFileSaveAs(uNotifyCode, nID, hWndCtl);
          } else {
            OnFileSave(uNotifyCode, nID, hWndCtl);
          }
        }
      } else { // IDCANCELL
        return ;
      }
    }

    CFileDialog dlg(TRUE, _T("txt"), NULL, OFN_HIDEREADONLY | OFN_CREATEPROMPT,
                    _T("eLXg t@C (*.txt)\0*.txt\0ׂẴt@C (*.*)\0*.*\0\0"));

    if(dlg.DoModal() == IDOK){
      if (FAILED(doLoadFile(dlg.m_szFileName))) {
        return ;
      }
      m_viewRichEdit.SetModify(FALSE);
      m_filename = dlg.m_szFileName;
    }

    return ;
  }

  static DWORD CALLBACK procWriteFile(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb)
  {
    if (::WriteFile(reinterpret_cast<HANDLE>(dwCookie), pbBuff, cb, reinterpret_cast<LPDWORD>(pcb), NULL)) {
      return FALSE;
    } else {
      return TRUE;
    }
  }

  BOOL doGetFileName(void)
  {
    BOOL bRet = FALSE;
    CFileDialog dlg(FALSE, _T("txt"), _T(""), OFN_OVERWRITEPROMPT,
      _T("eLXg t@C (*.txt)\0*.txt\0ׂẴt@C (*.*)\0*.*\0\0"));

    if(dlg.DoModal() == IDOK){
      m_filename = dlg.m_szFileName;
      _ASSERT(!m_filename.empty());
      bRet = TRUE;
    } else {
      bRet = FALSE;
    }
    return bRet;
  }

  HRESULT doSaveFile(LPCSTR lpcszFileName)
  {
    HANDLE hFile = ::CreateFile(lpcszFileName, GENERIC_WRITE, 0,
                        NULL, CREATE_ALWAYS,
                        FILE_ATTRIBUTE_NORMAL | FILE_FLAG_WRITE_THROUGH, NULL);
    if (hFile == INVALID_HANDLE_VALUE) {
      return HRESULT_FROM_WIN32(::GetLastError());
    }
    EDITSTREAM es;
    es.dwCookie = reinterpret_cast<DWORD>(hFile);
    es.dwError = 0;
    es.pfnCallback = procWriteFile;
    m_viewRichEdit.StreamOut(SF_TEXT, es);
    _ASSERT(!es.dwError);
    ::CloseHandle(hFile);
    return S_OK;
  }

  void OnFileSave(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    if (m_filename.empty()) {
      if (!doGetFileName()) return ;
    }

    doSaveFile(m_filename.c_str());
    m_viewRichEdit.SetModify(FALSE);

    return;
  }

  void OnFileSaveAs(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    if (doGetFileName()) {
      doSaveFile(m_filename.c_str());
      m_viewRichEdit.SetModify(FALSE);
    }

    return ;
  }

  void OnEditCut(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    m_viewRichEdit.Cut();
    return ;
  }

  void OnEditCopy(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    m_viewRichEdit.Copy();
    return ;
  }

  void OnEditPaste(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    m_viewRichEdit.Paste();
    return ;
  }

  void OnEditUndo(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    if (m_viewRichEdit.CanUndo()) {
      m_viewRichEdit.Undo();
    }
    return ;
  }

  void OnEditRedo(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    if (m_viewRichEdit.CanRedo()) {
      m_viewRichEdit.Redo();
    }
    return ;
  }

#ifdef FINDREPLACE_ENABELD
  void OnEditFind(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    if (m_ffindreplaceopen) {
      return ;
    }
    m_ffindreplaceopen = TRUE;
    CFindReplaceDialog *dlg = new CFindReplaceDialog;
    dlg->Create(TRUE, m_findstr.c_str(), NULL, FR_HIDEWHOLEWORD | FR_HIDEMATCHCASE | FR_HIDEUPDOWN);
    dlg->ShowWindow(SW_SHOW);
    return ;
  }

  void OnEditReplace(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    if (m_ffindreplaceopen) {
      return ;
    }
    m_ffindreplaceopen = TRUE;
    CFindReplaceDialog *dlg = new CFindReplaceDialog;
    dlg->Create(FALSE, m_findstr.c_str(), m_replacestr.c_str(), FR_HIDEWHOLEWORD | FR_HIDEMATCHCASE | FR_HIDEUPDOWN);
    dlg->ShowWindow(SW_SHOW);
    return ;
  }

  LRESUL OnFindReplace(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled)
  {
    bHandled = TRUE;
    CFindReplaceDialog* dlg = CFindReplaceDialog::GetNotifier(lParam);

    if (dlg->IsTerminating()) {
      m_ffindreplaceopen = FALSE;
      return 0;
    }

    if (dlg->FindNext()) {
      long nStartPos = 0, nEndPos = 0;
      FINDTEXTEX ftx;
      ::ZeroMemory(&ftx, sizeof(ftx));
      CString strFind = dlg->GetFindString();
      m_findstr = strFind;

      m_viewRichEdit.GetSel(nStartPos, nEndPos);
      ftx.chrg.cpMin = nEndPos;
      ftx.chrg.cpMax = -1;
      ftx.lpstrText = m_findstr.c_str();
      int nFindPos = m_viewRichEdit.FindText(FR_DOWN, ftx);
      if (nFindPos != -1) {
        m_viewRichEdit.SetSel(nFindPos, nFindPos + strFind.GetLength());
      }
    } else if (dlg->ReplaceCurrent()) {
      long nStartPos = 0, nEndPos = 0;
      FINDTEXTEX ftx;
      ::ZeroMemory(&ftx, sizeof(ftx));
      CString strFind = dlg->GetFindString();
      CString strReplace = dlg->GetReplaceString();
      m_findstr = strFind;
      m_replacestr = strReplace;
 
      m_viewRichEdit.GetSel(nStartPos, nEndPos);
//      long nGap = 0;
      if (nEndPos - nStartPos > 0) {
        m_viewRichEdit.ReplaceSel(dlg->GetReplaceString());
//        nGap = strReplace.GetLength() - strFind.GetLength();
      }
      ftx.chrg.cpMin = nEndPos;
      ftx.chrg.cpMax = -1;
      ftx.lpstrText = strFind;
      int nFindPos = m_viewRichEdit.FindText(FR_DOWN, ftx);
      if (nFindPos != -1) {
        m_viewRichEdit.SetSel(nFindPos, nFindPos + strFind.GetLength());
      }
    } else if (dlg->ReplaceAll()) {
      long nFindPos = 0, nFindCount = 0, nStartPos = 0;
      FINDTEXTEX ftx;
      ::ZeroMemory(&ftx, sizeof(ftx));
      CString strFind = dlg->GetFindString();
      CString strReplace = dlg->GetReplaceString();
      m_findstr = strFind;
      m_replacestr = strReplace;

//      long nGap = strReplace.GetLength() - strFind.GetLength();
      m_viewRichEdit.GetSel(nStartPos, nFindPos);
      ftx.chrg.cpMin = nFindPos;
      ftx.chrg.cpMax = -1;
      ftx.lpstrText = strFind;
      while ((nFindPos = m_viewRichEdit.FindText(FR_DOWN, ftx)) != -1) {
        m_viewRichEdit.SetSel(nFindPos, nFindPos + strFind.GetLength());
        m_viewRichEdit.ReplaceSel(strReplace);
        nFindCount++;
        nFindPos += strFind.GetLength();
        ftx.chrg.cpMin = nFindPos;
      }
    }

    return 0;
  }
#endif

  void OnEditSelAll(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    m_viewRichEdit.SetSel(0, -1);
    return ;
  }

  void OnEditExtEditor(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    std::string exteditor = m_exteditor;
    if (exteditor.empty()) {
      CHAR szDummyFileName[MAX_PATH + 1];
      CHAR szEditorFileName[MAX_PATH + 1];
      ::GetModuleFileName(NULL, szDummyFileName, MAX_PATH);
      *(::PathFindFileName(szDummyFileName)) = '\0';
      ::lstrcat(szDummyFileName, DUMMY_TEXT_FILENAME);
      
      HINSTANCE hInstance = ::FindExecutable(szDummyFileName, NULL, szEditorFileName);
      if (reinterpret_cast<int>(hInstance) <= 32) {
        // .txtɊ֘Atꂽt@CȂꍇ͉Ȃ
        return ;
      }
      exteditor = szEditorFileName;
    }

    HRESULT hr = S_OK;

    // ꎞt@C쐬
    CHAR szTempPath[MAX_PATH + 1];
    CHAR szTempFileName[MAX_PATH + 1];
    ::GetTempPath(MAX_PATH, szTempPath);
    if (!::GetTempFileName(szTempPath, "DZF", 0, szTempFileName)) {
      return ;
    }

    // ꎞt@C֏o
    if (FAILED(hr = doSaveFile(szTempFileName))) {
      ::DeleteFile(szTempFileName);
      return ;
    }

    // qXbhp̃pCv쐬
    CChildProcess cp;

    // R}hC쐬
    {
      std::string commandline = std::string("") + "\"" + exteditor + "\" \"" + szTempFileName + "\"";
      if (FAILED(cp.setCommandLine(commandline.c_str()))) {
        ::DeleteFile(szTempFileName);
        return ;
      }
    }

    // JgfBNg쐬
    if (FAILED(cp.setCurrentDirectory(szTempPath))) {
      ::DeleteFile(szTempFileName);
      return ;
    }

    // ϐ쐬i͕svj
    if (FAILED(cp.setEnvironment(NULL))) {
      ::DeleteFile(szTempFileName);
      return ;
    }

    if (FAILED(hr = cp.execute(TRUE))) {
      ::DeleteFile(szTempFileName);
      return ;
    }

    cp.waitForFinish(INFINITE);
    DWORD dwCode = cp.getExitCode();
    if (dwCode != 0) {
      ::DeleteFile(szTempFileName);
      return ;
    }

    doLoadFile(szTempFileName);
    ::DeleteFile(szTempFileName);

    return ;
  }

  void OnViewConvert(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    updateBrowser();
    return ;
  }
  
  LRESULT updateBrowser(void) {
    USES_CONVERSION;

    int iLen;
    CDzDocument dzd;
    HRESULT hr;

    iLen = m_viewRichEdit.GetTextLengthEx(GTL_USECRLF | GTL_PRECISE | GTL_NUMBYTES);

    LPSTR buff = new CHAR[iLen + 2];
    m_viewRichEdit.GetWindowText(buff, iLen + 1);
    buff[iLen] = '\0';
    hr = dzd.loadText(std::string(buff));
    delete [] buff;
    if (FAILED(hr)) return 0;

    std::string str;
    if (FAILED(hr = dzd.exportAsHTML(str))) {
      return 0;
    }
/*    
    CHAR szFileName[MAX_PATH + 1];
    ::GetModuleFileName(NULL, szFileName, MAX_PATH);
    *(::PathFindFileName(szFileName)) = '\0';
    ::lstrcat(szFileName, "test.html");
    HANDLE hFile = ::CreateFile(szFileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    DWORD dwWritten;
    ::WriteFile(hFile, str.c_str(), str.length(), &dwWritten, NULL);
    ::CloseHandle(hFile);
*/
    // uEUstr[h

    CComPtr<IDispatch> pDisp ;
    m_pBrowser->get_Document(&pDisp);
    CComQIPtr<IHTMLDocument3> pDoc = pDisp;
    CComQIPtr<IHTMLElement> pElement;
    if (SUCCEEDED(pDoc->getElementById(CComBSTR("contents"), &pElement))) {
      pElement->put_innerHTML(CComBSTR(str.c_str()));
    }

    return 0;
  }

  void OnViewToolBar(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    BOOL bVisible = !::IsWindowVisible(m_hWndToolBar);
    ::ShowWindow(m_hWndToolBar, bVisible ? SW_SHOWNOACTIVATE : SW_HIDE);
    UISetCheck(ID_VIEW_TOOLBAR, bVisible);
    UpdateLayout();
    return ;
  }

  void OnViewStatusBar(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    BOOL bVisible = !::IsWindowVisible(m_hWndStatusBar);
    ::ShowWindow(m_hWndStatusBar, bVisible ? SW_SHOWNOACTIVATE : SW_HIDE);
    UISetCheck(ID_VIEW_STATUS_BAR, bVisible);
    UpdateLayout();
    return ;
  }

  void OnOptionExtEditor(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    CFileDialog dlg(TRUE, _T("exe"), NULL, OFN_FILEMUSTEXIST,
                    _T("exe t@C (*.exe)\0*.exe\0ׂẴt@C (*.*)\0*.*\0\0"));
    if (dlg.DoModal() == IDOK) {
      m_exteditor = dlg.m_szFileName;
      CHAR szFileName[MAX_PATH + 1];
      ::GetModuleFileName(NULL, szFileName, MAX_PATH);
      *(::PathFindFileName(szFileName)) = '\0';
      ::lstrcat(szFileName, EXT_EDITOR_CONFIG_FILENAME);
      HANDLE hFile = ::CreateFile(szFileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
      if (hFile != INVALID_HANDLE_VALUE) {
        DWORD dwWritten;
        ::WriteFile(hFile, dlg.m_szFileName, ::lstrlen(dlg.m_szFileName), &dwWritten, NULL);
        ::CloseHandle(hFile);
      }
    }
    return ;
  }

  void OnOptionAutoPreview(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    m_autopreview = !m_autopreview;
    UISetCheck(ID_OPTION_AUTO_PREVIEW, m_autopreview ? 1 : 0);
    return ;
  }

  void OnHelpContent(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    CHAR szFileName[MAX_PATH + 1];
    LPSTR p;
    ::GetModuleFileName(NULL, szFileName, MAX_PATH);
    if ((p = ::PathFindFileName(szFileName)) != szFileName) {
      *p = '\0';
      ::lstrcat(szFileName, "manual.chm");
      ::HtmlHelp(m_hWnd, szFileName, HH_DISPLAY_TOPIC, 0);
    }
    return ;
  }

  void OnAppAbout(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    CAboutDlg dlg;
    dlg.DoModal();
    return ;
  }

  void OnChange(UINT uNotifyCode, int nID, HWND hWndCtl)
  {
    m_fchanged = TRUE;
    return ;
  }

  static unsigned __stdcall procReadStdErr(void *a) {
    CPipeThread *cpt = (CPipeThread *)a;
    HANDLE hRead = cpt->get_hParent();
    CHAR lpBuff[MAX_PATH + 1];
    DWORD dwRead = 0;
    std::string strerr;

    while (::ReadFile(hRead, lpBuff, MAX_PATH, &dwRead, NULL)) {
      *(lpBuff + dwRead) = '\0';
      strerr += lpBuff;
    }
    DWORD dwErr = ::GetLastError();
    if ((dwErr != ERROR_HANDLE_EOF) &&
      (dwErr != ERROR_BROKEN_PIPE)) {
      return -1;
    }
    if (strerr.length() > 0) {
      HANDLE hWnd = static_cast<HANDLE>(cpt->arg);
      // TODO:HWNDݒł悤
      ::MessageBox(NULL, strerr.c_str(), "Message", MB_OK);
    }
    cpt->close_hParent();
    return 0;
  }

  LRESULT OnCommand(UINT uMsg, WPARAM wParam, LPARAM lParam)
  {
    for (std::vector<CDzScript>::iterator itr = m_scripts.begin(); itr != m_scripts.end(); ++itr) {
      if ((*itr).get_script_id() == LOWORD(wParam)) {
        HRESULT hr = S_OK;

        // o̓t@C쐬
        std::string output_filename;
        {
          CHAR buff[1024];
          std::string ext = (*itr).get_script_ext();
          ::wsprintf(buff, "%s t@C (*.%s)\n*.%s\nׂẴt@C (*.*)\n*.*\n\n", ext.c_str(), ext.c_str(), ext.c_str());
          int len = ::lstrlen(buff);
          for (int i = 0; i < len; ++i) {
            if (buff[i] == '\n') buff[i] = '\0';
          }
//          std::string strfilter = ext + " t@C (*." + ext + ")\0*." + ext + "\0ׂẴt@C (*.*)\0*.*\0\0";
          CFileDialog dlg(FALSE, ext.c_str(), _T(""), OFN_OVERWRITEPROMPT,
            buff);

          if(dlg.DoModal() != IDOK) {
            return -1;
          }

          output_filename = dlg.m_szFileName;
          
          if (m_filename == output_filename) {
            MessageBox("et@C̏㏑͂ł܂", "Error", MB_OK);
            return -1;
          }
        }

        // o͗pXML쐬
        std::string strxml;
        {
          int iLen;
          CDzDocument dzd;
          HRESULT hr;

          iLen = m_viewRichEdit.GetTextLengthEx(GTL_USECRLF | GTL_PRECISE | GTL_NUMBYTES);

          LPSTR buff = new CHAR[iLen + 2];
          m_viewRichEdit.GetWindowText(buff, iLen + 1);
          buff[iLen] = '\0';
          hr = dzd.loadText(std::string(buff));
          delete [] buff;
          if (FAILED(hr)) return -1;

          if (FAILED(hr = dzd.exportAsXML(strxml))) {
            return -1;
          }
/*
          CHAR szFileName[MAX_PATH + 1];
          ::GetModuleFileName(NULL, szFileName, MAX_PATH);
          *(::PathFindFileName(szFileName)) = '\0';
          ::lstrcat(szFileName, "test.xml");
          HANDLE hFile = ::CreateFile(szFileName, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
          DWORD dwWritten;
          ::WriteFile(hFile, strxml.c_str(), strxml.length(), &dwWritten, NULL);
          ::CloseHandle(hFile);
*/
        }

        // qXbhp̃pCv쐬
        CChildProcess cp;

        CPipeThread se;
        if (FAILED(hr = se.prepare_read())) return -1;
        cp.setStdErr(se.get_hChild());

        CPipeThread si;
        if (FAILED(hr = si.prepare_write())) return -1;
        cp.setStdIn(si.get_hChild());

        // script̃fBNgm
        std::string script_directory;
        {
          CHAR szFileName[MAX_PATH + 1];
          ::GetModuleFileName(NULL, szFileName, MAX_PATH);
          *(::PathFindFileName(szFileName)) = '\0';
          script_directory = szFileName;
          script_directory.append("script\\");
        }

        // R}hC쐬
        {
          std::string script_filename = script_directory + (*itr).get_script_name();
          std::string commandline = (*itr).get_script_host() + " \"" + script_filename + "\" " + (*itr).get_script_option() + " -o \""+ output_filename + "\"";
          if (FAILED(cp.setCommandLine(commandline.c_str()))) return -1;
        }

        // JgfBNg쐬
        if (FAILED(cp.setCurrentDirectory(script_directory.c_str()))) return -1;;

        // ϐ쐬i͕svj
        if (FAILED(cp.setEnvironment(NULL))) return -1;;

        if (FAILED(hr = cp.execute(FALSE))) return -1;
        se.close_hChild();
        si.close_hChild();

        se.arg = static_cast<void *>(m_hWnd);

        // qvZXN
        HANDLE he = se.execute(procReadStdErr);
        if (he == (HANDLE)-1) return -1;
        {
          DWORD dwWritten;
          ::WriteFile(si.get_hParent(), strxml.c_str(), strxml.length(), &dwWritten, NULL);
          si.close_hParent();
        }
        ::WaitForSingleObject(he, INFINITE);
        DWORD dwCode = se.getExitCode();
        if (dwCode != 0) return -1;

        cp.waitForFinish(INFINITE);
        dwCode = cp.getExitCode();
        if (dwCode != 0) return -1;

        // vr[ʕ\
        ::ShellExecute(NULL, "open", output_filename.c_str(), NULL, NULL, SW_SHOW);

        return 0;
      }
    }
    return -1;
  }

};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ ͑Os̒Oɒǉ̐錾}܂B

#endif // !defined(AFX_MAINFRM_H__07CD2330_35DA_4260_8287_79CBFE15C4E0__INCLUDED_)
